package com.mortals.xhx.module.record.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.CertificateStatus;
import com.mortals.xhx.common.code.GenerateStatus;
import com.mortals.xhx.common.code.PrintStatus;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.ParamKey;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.certificate.model.CertificateCatalogEntity;
import com.mortals.xhx.module.certificate.model.CertificateClassifyQuery;
import com.mortals.xhx.module.certificate.service.CertificateCatalogService;
import com.mortals.xhx.module.certificate.service.CertificateClassifyService;
import com.mortals.xhx.module.record.dao.PrintLogDao;
import com.mortals.xhx.module.record.dao.PrintWaitQueueDao;
import com.mortals.xhx.module.record.dao.RetainLogDao;
import com.mortals.xhx.module.record.model.*;
import com.mortals.xhx.module.record.service.ApplyLogService;
import com.mortals.xhx.module.record.service.PrintWaitQueueService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
* PrintWaitQueueService
* 证照打印记录 service实现
*
* @author zxfei
* @date 2022-10-22
*/
@Service("printWaitQueueService")
public class PrintWaitQueueServiceImpl extends AbstractCRUDServiceImpl<PrintWaitQueueDao, PrintWaitQueueEntity, Long> implements PrintWaitQueueService {

    /** 系统ip端口 */
    @Value("${system.ip:http://192.168.0.98:11075/}")
    private String ip;
    /** 打印api接口地址 */
    @Value("${print.aip:http://192.168.0.58:8037/api/print/GetHardWareInfo?GetType=78}")
    private String printApi;
    /** 打印机设备ip */
    @Value("${system.print:192.168.0.11}")
    private String printIp;

    @Autowired
    private PrintLogDao printLogDao;
    @Autowired
    private RetainLogDao retainLogDao;
    @Autowired
    private CertificateCatalogService certificateCatalogService;
    @Autowired
    private CertificateClassifyService certificateClassifyService;
    @Autowired
    private ParamService paramService;
    @Autowired
    private ApplyLogService applyLogService;

    @Override
    protected void findAfter(PrintWaitQueueEntity entity, Context context, List<PrintWaitQueueEntity> list) throws AppException {

        for(PrintWaitQueueEntity printWaitQueueEntity:list){
            CertificateCatalogEntity catalogEntity = certificateCatalogService.get(printWaitQueueEntity.getCatalogId());
            printWaitQueueEntity.setPaperSource(catalogEntity.getPaperSource());
            printWaitQueueEntity.setPrinterIp(catalogEntity.getPrinterIp());
            printWaitQueueEntity.setPrinterName(catalogEntity.getPrinterName());
            printWaitQueueEntity.setPrinterServiceUrl("http://127.0.0.1:8037");
        }
    }

    @Override
    protected void findAfter(PrintWaitQueueEntity entity, PageInfo pageInfo, Context context, List<PrintWaitQueueEntity> list) throws AppException {

        for(PrintWaitQueueEntity printWaitQueueEntity:list){
            CertificateCatalogEntity catalogEntity = certificateCatalogService.get(printWaitQueueEntity.getCatalogId());
            printWaitQueueEntity.setPaperSource(catalogEntity.getPaperSource());
            printWaitQueueEntity.setPrinterIp(catalogEntity.getPrinterIp());
            printWaitQueueEntity.setPrinterName(catalogEntity.getPrinterName());
            printWaitQueueEntity.setPrinterServiceUrl("http://127.0.0.1:8037");
        }
    }

    @Override
    public void creatWaitQueueByApply(ApplyLogEntity applyLogEntity, int generateType, Context context) throws AppException {
        //CertificateCatalogEntity catalogEntity = certificateCatalogService.get(applyLogEntity.getCatalogId());
        if(generateType== GenerateStatus.ALL.getValue()){
            PrintWaitQueueEntity original = creatWaitQueue(applyLogEntity,GenerateStatus.ORIGINAL.getValue());
            dao.insert(original);
            PrintWaitQueueEntity duplicate = creatWaitQueue(applyLogEntity,GenerateStatus.DUPLICATE.getValue());
            dao.insert(duplicate);
        }else {
            PrintWaitQueueEntity printWaitQueueEntity = creatWaitQueue(applyLogEntity,generateType);
            dao.insert(printWaitQueueEntity);
        }
    }

    private PrintWaitQueueEntity creatWaitQueue(ApplyLogEntity applyLogEntity,int generateType){
        PrintWaitQueueEntity waitQueueEntity = new PrintWaitQueueEntity();
        BeanUtils.copyProperties(applyLogEntity, waitQueueEntity, BeanUtil.getNullPropertyNames(applyLogEntity));
        waitQueueEntity.setId(null);
        waitQueueEntity.setUpdateUserId(null);
        waitQueueEntity.setUpdateTime(null);
        waitQueueEntity.setApplyId(applyLogEntity.getId());
        waitQueueEntity.setPrintStatus(YesNoEnum.NO.getValue());
        waitQueueEntity.setTotal(paramService.getParamIntValue(ParamKey.ALLOW_PRINT_MAX_COUNT));
        waitQueueEntity.setCreateUserId(applyLogEntity.getCreateUserId());
        waitQueueEntity.setCreateTime(new Date());
        return waitQueueEntity;
    }

    @Override
    public void doPrint(Long id,int printStatus,String statusRemark) {
        PrintWaitQueueEntity waitQueueEntity = this.dao.get(id);
        if(waitQueueEntity==null){
            throw new AppException("数据不存在或已修改");
        }
        PrintWaitQueueQuery data = new PrintWaitQueueQuery();
        data.setId(id);
        if(printStatus==PrintStatus.SUCCESS.getValue()){
            int total = waitQueueEntity.getTotal();
            total--;
            doPrintSuccess(waitQueueEntity);
            if(total==0){
                data.setPrintStatus(YesNoEnum.YES.getValue());
            }
            data.setTotalIncrement(-1);
        }else {
            doPrintFail(waitQueueEntity,statusRemark);
            //data.setPrintStatus(-1);
        }
        data.setUpdateTime(new Date());
        dao.update(data);

    }

    private void doPrintFail(PrintWaitQueueEntity waitQueueEntity,String statusRemark){
        Date now = new Date();
        PrintLogEntity printLogEntity = new PrintLogEntity();
        BeanUtils.copyProperties(waitQueueEntity, printLogEntity, BeanUtil.getNullPropertyNames(waitQueueEntity));
        printLogEntity.setId(null);
        printLogEntity.setUpdateUserId(null);
        printLogEntity.setUpdateTime(null);
        printLogEntity.setPrintDate(now);
        printLogEntity.setCreateUserId(waitQueueEntity.getCreateUserId());
        printLogEntity.setCreateTime(now);
        printLogEntity.setPrintStatus(PrintStatus.FAIL.getValue());
        printLogEntity.setStatusRemark(statusRemark);
        printLogEntity.setMobile(waitQueueEntity.getMobile());
        printLogDao.insert(printLogEntity);
    }

    private void doPrintSuccess(PrintWaitQueueEntity waitQueueEntity){

        ApplyLogEntity applyLogEntity = applyLogService.get(waitQueueEntity.getApplyId());
        Date now = new Date();


        PrintLogEntity printLogEntity = new PrintLogEntity();
        BeanUtils.copyProperties(waitQueueEntity, printLogEntity, BeanUtil.getNullPropertyNames(waitQueueEntity));
        printLogEntity.setId(null);
        printLogEntity.setUpdateUserId(null);
        printLogEntity.setUpdateTime(null);
        printLogEntity.setPrintDate(now);
        printLogEntity.setCreateUserId(waitQueueEntity.getCreateUserId());
        printLogEntity.setCreateTime(now);
        printLogEntity.setPrintStatus(PrintStatus.SUCCESS.getValue());
        printLogEntity.setStatusRemark(PrintStatus.SUCCESS.getDesc());
        RetainLogEntity retainLogquery = new RetainLogEntity();
        retainLogquery.setApplyId(waitQueueEntity.getApplyId());
        List<RetainLogEntity> tempRetainLogList = retainLogDao.getList(retainLogquery);
        if(CollectionUtils.isNotEmpty(tempRetainLogList)){
            RetainLogQuery updateRetainLog = new RetainLogQuery();
            updateRetainLog.setUpdateTime(now);
            updateRetainLog.setId(tempRetainLogList.get(0).getId());
            updateRetainLog.setTotalIncrement(1);
            retainLogDao.update(updateRetainLog);
        }else {
            RetainLogEntity retainLogEntity = new RetainLogEntity();
            BeanUtils.copyProperties(applyLogEntity, retainLogEntity, BeanUtil.getNullPropertyNames(applyLogEntity));
            retainLogEntity.setId(null);
            retainLogEntity.setUpdateUserId(null);
            retainLogEntity.setUpdateTime(null);

            retainLogEntity.setCertificateStatus(CertificateStatus.NORMAL.getValue());

            retainLogEntity.setCreateUserId(waitQueueEntity.getCreateUserId());
            retainLogEntity.setCreateTime(now);
            retainLogDao.insert(retainLogEntity);
        }
        printLogDao.insert(printLogEntity);
        CertificateCatalogEntity catalogEntity = certificateCatalogService.get(waitQueueEntity.getCatalogId());
        CertificateClassifyQuery query = new CertificateClassifyQuery();
        query.setId(catalogEntity.getClassifyId());
        query.setTotalIncrement(1);
        query.setUpdateTime(now);
        certificateClassifyService.update(query);
        ApplyLogEntity updateApply = new ApplyLogEntity();
        updateApply.setId(applyLogEntity.getId());
        updateApply.setCertificateCode(applyLogEntity.getCertificateCode());
        updateApply.setEnterpriseName(applyLogEntity.getEnterpriseName());
        updateApply.setCatalogId(applyLogEntity.getCatalogId());
        updateApply.setRecordStatus(YesNoEnum.YES.getValue());
        updateApply.setUpdateTime(new Date());
        applyLogService.update(updateApply);
    }
}